from __future__ import absolute_import, division, print_function

__version__ = "3.4.1"
dependencies = ["restnavigator==1.0.1",
                "requests[socks]==2.28.1",
                "configparser==5.3.0",
                "certifi==2022.9.24"]
